#include <lmic.h>
#include <hal/hal.h>
#include <SPI.h>

#include "settings.h"
#include "storage.h"

// Global static variable that's used to detect when LMIC has finished doing it's thing
bool is_sending_complete = false;

// Defined as part of LMIC, but LMIC is compiled and linked in separately
extern "C" void lmic_aes_encrypt(unsigned char *Data, unsigned char *Key);


void transmit_counter_save() {
	store_eeprom_uint32_save(0, LMIC.seqnoUp);
	store_eeprom_uint32_save(sizeof(uint32_t), LMIC.seqnoDn);
}


void transmit_init() {
	Serial.println(F("[LMIC] Init"));
	// LMIC init
	os_init();
	// Reset the MAC state. Session and pending data transfers will be discarded.
	LMIC_reset();
	
	// Set static session parameters. Instead of dynamically establishing a session
	// by joining the network, precomputed session parameters are be provided.
	#ifdef PROGMEM
	// On AVR, these values are stored in flash and only copied to RAM
	// once. Copy them to a temporary buffer here, LMIC_setSession will
	// copy them into a buffer of its own again.
	uint8_t appskey[sizeof(APPSKEY)];
	uint8_t nwkskey[sizeof(NWKSKEY)];
	memcpy_P(appskey, APPSKEY, sizeof(APPSKEY));
	memcpy_P(nwkskey, NWKSKEY, sizeof(NWKSKEY));
	LMIC_setSession (0x1, DEVADDR, nwkskey, appskey);
	#else
	// If not running an AVR with PROGMEM, just use the arrays directly 
	LMIC_setSession (0x1, DEVADDR, NWKSKEY, APPSKEY);
	#endif
	
	// Set up the channels used by the Things Network, which corresponds
	// to the defaults of most gateways. Without this, only three base
	// channels from the LoRaWAN specification are used, which certainly
	// works, so it is good for debugging, but can overload those
	// frequencies, so be sure to configure the full frequency range of
	// your network here (unless your network autoconfigures them).
	// Setting up channels should happen after LMIC_setSession, as that
	// configures the minimal channel set.
	LMIC_setupChannel(0, 868100000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(1, 868300000, DR_RANGE_MAP(DR_SF12, DR_SF7B), BAND_CENTI);      // g-band
	LMIC_setupChannel(2, 868500000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(3, 867100000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(4, 867300000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(5, 867500000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(6, 867700000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(7, 867900000, DR_RANGE_MAP(DR_SF12, DR_SF7),  BAND_CENTI);      // g-band
	LMIC_setupChannel(8, 868800000, DR_RANGE_MAP(DR_FSK,  DR_FSK),  BAND_MILLI);      // g2-band
	// TTN defines an additional channel at 869.525Mhz using SF9 for class B
	// devices' ping slots. LMIC does not have an easy way to define set this
	// frequency and support for class B is spotty and untested, so this
	// frequency is not configured here.

	// Disable link check validation
	LMIC_setLinkCheckMode(0);

	// Set data rate and transmit power (note: txpow seems to be ignored by the library)
	LMIC_setDrTxpow(DR_SF7,14);
	
	// Restore the frame counters
	LMIC.seqnoUp = store_eeprom_uint32_retrieve(0);
	LMIC.seqnoDn = store_eeprom_uint32_retrieve(sizeof(uint32_t));
	// Serial.println(LMIC.seqnoUp);
	if(LMIC.seqnoUp >= 4294967295) { // The default value
		LMIC.seqnoUp = LMIC.seqnoDn = 0; // Reset to 0
		transmit_counter_save();
	}
}

/**
 * Sends a specified message via LoRaWAN.
 * @param data   The message to send.
 * @param length The length of the given message.
 */
bool transmit_send(uint8_t* data, int length) {
	// Encrypt the data with the extra encryption key
	lmic_aes_encrypt(data, encrypt_key);
	
	// Check if there is not a current TX/RX job running
	if (LMIC.opmode & OP_TXRXPEND) {
		// Serial.println(F("OP_TXRXPEND: Job running, can't send"));
		return false;
	}
	// Prepare upstream data transmission at the next possible time.
	LMIC_setTxData2(1, data, length, 0);
	Serial.println(F("done"));
	
	// Run the LMIC loop, but only until it's finished sending the packet
	while (!is_sending_complete) {
		os_runloop_once();
	}
	
	// Persist the frame counters
	transmit_counter_save();
	
	// Reset it for next time (just in case)
	is_sending_complete = false;
	
	return true;
}

void onEvent (ev_t ev) {
	Serial.print(millis());
	Serial.print(" ");
	switch(ev) {
		/*case EV_SCAN_TIMEOUT:
			Serial.println(F("EV_SCAN_TIMEOUT"));
			break;
		case EV_BEACON_FOUND:
			Serial.println(F("EV_BEACON_FOUND"));
			break;
		case EV_BEACON_MISSED:
			Serial.println(F("EV_BEACON_MISSED"));
			break;
		case EV_BEACON_TRACKED:
			Serial.println(F("EV_BEACON_TRACKED"));
			break;
		case EV_JOINING:
			Serial.println(F("EV_JOINING"));
			break;
		case EV_JOINED:
			Serial.println(F("EV_JOINED"));
			break;
		case EV_RFU1:
			Serial.println(F("EV_RFU1"));
			break;
		case EV_JOIN_FAILED:
			Serial.println(F("EV_JOIN_FAILED"));
			break;
		case EV_REJOIN_FAILED:
			Serial.println(F("EV_REJOIN_FAILED"));
			break;*/
		case EV_TXCOMPLETE:
			Serial.println(F("EV_TXCOMPLETE incl. RX"));
			if(LMIC.dataLen) {
				// data received in rx slot after tx
				Serial.print(F("Received: "));
				Serial.write(LMIC.frame+LMIC.dataBeg, LMIC.dataLen);
				Serial.println();
			}
			// We're done!
			is_sending_complete = true;
			break;
		/*case EV_LOST_TSYNC:
			Serial.println(F("EV_LOST_TSYNC"));
			break;
		case EV_RESET:
			Serial.println(F("EV_RESET"));
			break;*/
		case EV_RXCOMPLETE:
			// data received in ping slot
			Serial.println(F("EV_RXCOMPLETE"));
			break;
		/*case EV_LINK_DEAD:
			Serial.println(F("EV_LINK_DEAD"));
			break;
		case EV_LINK_ALIVE:
			Serial.println(F("EV_LINK_ALIVE"));
			break;*/
		default:
			Serial.print(F("Unknown event "));
			Serial.println(ev);
			break;
	}
}
